/*	Application		Periodic Output
	File			PeriodicOutput.cpp 
	Purpose			Demonstration of command-line configurable program, with periodic output
	Author			Richard John Anthony
	Date			December 2013
	
	Function	1. Reads command-line parameters which configure the program's behaviour
				2. Outputs a user-specified character
				3. Sleeps a user-defined period
				4. Repeats in loop for as many iterations as necessary until the total run time reaches 10 seconds
*/

#include "stdafx.h"
#include <iostream>
#include <Windows.h>

using namespace std;

int main(int argc, char* argv[])
{
	int iInterval;
	int iTotalRunTime_Milliseconds = 10000;		// Program will run for 10 seconds
	int iAccumulatedRunTime_Milliseconds = 0;	// Run time accumulated
	char cCharToDisplay;

	if(3 != argc)
	{
		// User did not provide the required parameters
		cout << "Usage: " << argv[0] << " milliseconds_period character_to_print";
		return -1;
	}
	
	iInterval = atoi(argv[1]);
	if(0 >= iInterval)
	{
		// iInterval too small (less than or equal to 0)
		cout << "Interval must be greater or equal to 1 millisecond";
		return -1;
	}
	
	cCharToDisplay = argv[2][0];	// Get the first of possibly many chars that constitute the second argument

	while(iAccumulatedRunTime_Milliseconds < iTotalRunTime_Milliseconds)
	{
		cout << cCharToDisplay;
		Sleep(iInterval);
		iAccumulatedRunTime_Milliseconds += iInterval;
	}

	return 0;
}

