/*	Application		Threads Demonstration - Threads executed asynchronously		
	File			Threads_Detach.cpp
	Purpose			Demonstrates the basic concepts of multi-threading
	Author			Richard John Anthony
	Date			January 2014
	
	Function		1. Creates three threads
					2. Runs the threads asynchronously - uses detach()
					3. Threads display text messages for diagnostic purposes
*/

#include <string>
#include <iostream>
#include <thread>  // using the std::thread class

using namespace std;

void Function1(string sStr)
{
	size_t tThreadID = std::this_thread::get_id().hash();

	for (int iCount = 0; iCount < 10; iCount++)
	{
		cout << "Function 1: " << tThreadID << " " << sStr << endl;
	}
}

void Function2(string sStr)
{
	size_t tThreadID = std::this_thread::get_id().hash();

	for (int iCount = 0; iCount < 10; iCount++)
	{
		cout << "Function 2: " << tThreadID << " " << sStr << endl;
	}
}

void Function3(string sStr)
{
	size_t tThreadID = std::this_thread::get_id().hash();

	for (int iCount = 0; iCount < 10; iCount++)
	{
		cout << "Function 3: " << tThreadID << " " << sStr << endl;
	}
}

int main()
{
	char cWaitForUser;
    // Create three threads
    thread thread1(Function1 /*function the thread will perform*/, "AAAAAAAAAAAA" /*parameter for thread's function*/);
    thread thread2(Function2 /*function the thread will perform*/, "BBBBBBBBBBBB" /*parameter for thread's function*/);
    thread thread3(Function3 /*function the thread will perform*/, "CCCCCCCCCCCC" /*parameter for thread's function*/);

    // Detach the threads
	// Using detach() causes the threads to run asynchronously with each other
	// This means that the three threads and main all run 'concurrently' (and could run in true parallel on a four core processor)
	thread1.detach();    
	thread2.detach();    
	thread3.detach();

	for (int iCount = 0; iCount < 10; iCount++)
	{
		cout << "Main" << endl;
	}

	cout << "To Exit press any key, followed by the Enter key " << endl;
	cin >> cWaitForUser;
}

